@@ -22,19 +22,19 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/mman.h>
 
+#include <spa/utils/string.h>
+
 #include <jack/metadata.h>
 #include <jack/uuid.h>
 
 #include <pipewire/pipewire.h>
-#include <extensions/metadata.h>
+#include <pipewire/extensions/metadata.h>
 
 static jack_description_t *find_description(jack_uuid_t subject)
 {
@@ -53,6 +53,13 @@ static void set_property(jack_property_t *prop, const char *key, const char *val
 	prop->type = strdup(type);
 }
 
+static void clear_property(jack_property_t *prop)
+{
+	free((char*)prop->key);
+	free((char*)prop->data);
+	free((char*)prop->type);
+}
+
 static jack_property_t *copy_properties(jack_property_t *src, uint32_t cnt)
 {
 	jack_property_t *dst;
@@ -98,7 +105,7 @@ static jack_property_t *find_property(jack_description_t *desc, const char *key)
 	uint32_t i;
 	for (i = 0; i < desc->property_cnt; i++) {
 		jack_property_t *prop = &desc->properties[i];
-		if (strcmp(prop->key, key) == 0)
+		if (spa_streq(prop->key, key))
 			return prop;
 	}
 	return NULL;
@@ -108,53 +115,43 @@ static jack_property_t *add_property(jack_description_t *desc, const char *key,
 		const char *value, const char *type)
 {
 	jack_property_t *prop;
+	void *np;
+	size_t ns;
 
 	if (desc->property_cnt == desc->property_size) {
-		desc->property_size = desc->property_size > 0 ? desc->property_size * 2 : 8;
-		desc->properties = realloc(desc->properties, sizeof(*prop) * desc->property_size);
+		ns = desc->property_size > 0 ? desc->property_size * 2 : 8;
+		np = pw_reallocarray(desc->properties, ns, sizeof(*prop));
+		if (np == NULL)
+			return NULL;
+		desc->property_size = ns;
+		desc->properties = np;
 	}
 	prop = &desc->properties[desc->property_cnt++];
 	set_property(prop, key, value, type);
 	return prop;
 }
 
-static void clear_property(jack_property_t *prop)
-{
-	free((char*)prop->key);
-	free((char*)prop->data);
-	free((char*)prop->type);
-}
-
 static void remove_property(jack_description_t *desc, jack_property_t *prop)
 {
 	clear_property(prop);
 	desc->property_cnt--;
-        memmove(desc->properties, SPA_MEMBER(prop, sizeof(*prop), void),
-                SPA_PTRDIFF(SPA_MEMBER(desc->properties, sizeof(*prop) * desc->property_cnt, void),
+        memmove(desc->properties, SPA_PTROFF(prop, sizeof(*prop), void),
+                SPA_PTRDIFF(SPA_PTROFF(desc->properties, sizeof(*prop) * desc->property_cnt, void),
 			prop));
 
 	if (desc->property_cnt == 0)
 		remove_description(desc);
 }
 
-static inline int strzcmp(const char *s1, const char *s2)
-{
-	if (s1 == s2)
-		return 0;
-	if (s1 == NULL || s2 == NULL)
-		return 1;
-	return strcmp(s1, s2);
-}
-
 static int change_property(jack_property_t *prop, const char *value, const char *type)
 {
 	int changed = 0;
-	if (strzcmp(prop->data, value) != 0) {
+	if (!spa_streq(prop->data, value)) {
 		free((char*)prop->data);
 		prop->data = strdup(value);
 		changed++;
 	}
-	if (strzcmp(prop->type, type) != 0) {
+	if (!spa_streq(prop->type, type)) {
 		free((char*)prop->type);
 		prop->type = strdup(type);
 		changed++;
@@ -195,9 +192,16 @@ static int update_property(struct client *c,
 		} else if (prop == NULL) {
 			if (desc == NULL)
 				desc = add_description(subject);
-			prop = add_property(desc, key, value, type);
-			change = PropertyCreated;
-			changed++;
+			if (desc == NULL) {
+				changed = -errno;
+				pw_log_warn("add_description failed: %m");
+			} else if (add_property(desc, key, value, type) == NULL) {
+				changed = -errno;
+				pw_log_warn("add_property failed: %m");
+			} else {
+				change = PropertyCreated;
+				changed++;
+			}
 		} else {
 			changed = change_property(prop, value, type);
 			change = PropertyChanged;
@@ -205,10 +209,11 @@ static int update_property(struct client *c,
 	}
 	pthread_mutex_unlock(&globals.lock);
 
-	if (c->property_callback && changed)
+	if (c->property_callback && changed > 0) {
+		pw_log_info("emit %lu %s", subject, key);
 		c->property_callback(subject, key, change, c->property_arg);
-
-	return 0;
+	}
+	return changed;
 }
 
 
@@ -220,7 +225,8 @@ int jack_set_property(jack_client_t*client,
 		      const char* type)
 {
 	struct client *c = (struct client *) client;
-	uint32_t id;
+	struct object *o;
+	uint32_t serial;
 	int res = -1;
 
 	spa_return_val_if_fail(c != NULL, -EINVAL);
@@ -231,14 +237,19 @@ int jack_set_property(jack_client_t*client,
 	if (c->metadata == NULL)
 		goto done;
 
-	id = jack_uuid_to_index(subject);
+	if (subject & (1<<30))
+		goto done;
+
+	serial = jack_uuid_to_index(subject);
+	if ((o = find_by_serial(c, serial)) == NULL)
+		goto done;
 
 	if (type == NULL)
 		type = "";
 
-	pw_log_info("set id:%u (%"PRIu64") '%s' to '%s@%s'", id, subject, key, value, type);
-	update_property(c, id, key, type, value);
-	pw_metadata_set_property(c->metadata->proxy, id, key, type, value);
+	pw_log_info("set id:%u (%"PRIu64") '%s' to '%s@%s'", o->id, subject, key, value, type);
+	if (update_property(c, subject, key, type, value))
+		pw_metadata_set_property(c->metadata->proxy, o->id, key, type, value);
 	res = 0;
 done:
 	pw_thread_loop_unlock(c->context.loop);
