@@ -39,6 +39,7 @@ extern "C" {
 #include <spa/support/loop.h>
 #include <spa/utils/list.h>
 #include <spa/utils/json.h>
+#include <spa/utils/dll.h>
 
 #include <spa/node/node.h>
 #include <spa/node/utils.h>
@@ -50,11 +51,10 @@ extern "C" {
 
 #include "alsa.h"
 
-#include "dll.h"
 
-#define MIN_LATENCY	16
-#define MAX_LATENCY	8192
+#define MAX_RATES	16
 
+#define DEFAULT_PERIOD		1024u
 #define DEFAULT_RATE		48000u
 #define DEFAULT_CHANNELS	2u
 #define DEFAULT_USE_CHMAP	false
@@ -63,8 +63,6 @@ struct props {
 	char device[64];
 	char device_name[128];
 	char card_name[128];
-	uint32_t min_latency;
-	uint32_t max_latency;
 	bool use_chmap;
 };
 
@@ -108,7 +106,7 @@ struct state {
 	struct spa_system *data_system;
 	struct spa_loop *data_loop;
 
-	int card_index;
+	uint32_t card_index;
 	struct card *card;
 	snd_pcm_stream_t stream;
 	snd_output_t *output;
@@ -133,15 +131,19 @@ struct state {
 	struct spa_audio_info current_format;
 
 	uint32_t default_period_size;
+	uint32_t default_period_num;
 	uint32_t default_headroom;
 	uint32_t default_start_delay;
 	uint32_t default_format;
 	unsigned int default_channels;
 	unsigned int default_rate;
+	uint32_t allowed_rates[MAX_RATES];
+	uint32_t n_allowed_rates;
 	struct channel_map default_pos;
 	unsigned int disable_mmap;
 	unsigned int disable_batch;
 	char clock_name[64];
+	uint32_t quantum_limit;
 
 	snd_pcm_uframes_t buffer_frames;
 	snd_pcm_uframes_t period_frames;
@@ -149,6 +151,7 @@ struct state {
 	int rate;
 	int channels;
 	size_t frame_size;
+	size_t frame_scale;
 	int blocks;
 	uint32_t rate_denom;
 	uint32_t delay;
@@ -187,10 +190,9 @@ struct state {
 	uint32_t start_delay;
 
 	uint32_t duration;
-	uint32_t last_duration;
-	uint64_t last_position;
 	unsigned int alsa_started:1;
 	unsigned int alsa_sync:1;
+	unsigned int alsa_sync_warning:1;
 	unsigned int alsa_recovering:1;
 	unsigned int following:1;
 	unsigned int matching:1;
@@ -208,7 +210,6 @@ struct state {
 	int64_t sample_count;
 
 	int64_t sample_time;
-	uint64_t current_time;
 	uint64_t next_time;
 	uint64_t base_time;
 
@@ -283,6 +284,22 @@ static inline void spa_alsa_parse_position(struct channel_map *map, const char *
 	}
 }
 
+static inline uint32_t spa_alsa_parse_rates(uint32_t *rates, uint32_t max, const char *val, size_t len)
+{
+	struct spa_json it[2];
+	char v[256];
+	uint32_t count;
+
+	spa_json_init(&it[0], val, len);
+        if (spa_json_enter_array(&it[0], &it[1]) <= 0)
+                spa_json_init(&it[1], val, len);
+
+	count = 0;
+	while (spa_json_get_string(&it[1], v, sizeof(v)) > 0 && count < max)
+		rates[count++] = atoi(v);
+	return count;
+}
+
 static inline uint32_t spa_alsa_iec958_codec_from_name(const char *name)
 {
 	int i;
