@@ -25,6 +25,14 @@
 #include "fmt-ops.h"
 
 #include <immintrin.h>
+// GCC: workaround for missing AVX intrinsic: "_mm256_setr_m128()"
+//      (see https://stackoverflow.com/questions/32630458/setting-m256i-to-the-value-of-two-m128i-values)
+#ifndef _mm256_setr_m128i
+#  ifndef _mm256_set_m128i
+#    define _mm256_set_m128i(v0, v1)  _mm256_insertf128_si256(_mm256_castsi128_si256(v1), (v0), 1)
+#  endif
+#  define _mm256_setr_m128i(v0, v1) _mm256_set_m128i((v1), (v0))
+#endif
 
 static void
 conv_s16_to_f32d_1s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA_RESTRICT src,
@@ -33,7 +41,7 @@ conv_s16_to_f32d_1s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 	const int16_t *s = src;
 	float *d0 = dst[0];
 	uint32_t n, unrolled;
-	__m256i in;
+	__m256i in = _mm256_setzero_si256();
 	__m256 out, factor = _mm256_set1_ps(1.0f / S16_SCALE);
 
 	if (SPA_LIKELY(SPA_IS_ALIGNED(d0, 32)))
@@ -59,7 +67,7 @@ conv_s16_to_f32d_1s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 	}
 	for(; n < n_samples; n++) {
 		__m128 out, factor = _mm_set1_ps(1.0f / S16_SCALE);
-		out = _mm_cvtsi32_ss(out, s[0]);
+		out = _mm_cvtsi32_ss(factor, s[0]);
 		out = _mm_mul_ss(out, factor);
 		_mm_store_ss(&d0[n], out);
 		s += n_channels;
@@ -125,9 +133,9 @@ conv_s16_to_f32d_2_avx2(struct convert *conv, void * SPA_RESTRICT dst[], const v
 	}
 	for(; n < n_samples; n++) {
 		__m128 out[4], factor = _mm_set1_ps(1.0f / S16_SCALE);
-		out[0] = _mm_cvtsi32_ss(out[0], s[0]);
+		out[0] = _mm_cvtsi32_ss(factor, s[0]);
 		out[0] = _mm_mul_ss(out[0], factor);
-		out[1] = _mm_cvtsi32_ss(out[1], s[1]);
+		out[1] = _mm_cvtsi32_ss(factor, s[1]);
 		out[1] = _mm_mul_ss(out[1], factor);
 		_mm_store_ss(&d0[n], out[0]);
 		_mm_store_ss(&d1[n], out[1]);
@@ -167,7 +175,7 @@ conv_s24_to_f32d_1s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 		s += 12 * n_channels;
 	}
 	for(; n < n_samples; n++) {
-		out = _mm_cvtsi32_ss(out, read_s24(s));
+		out = _mm_cvtsi32_ss(factor, read_s24(s));
 		out = _mm_mul_ss(out, factor);
 		_mm_store_ss(&d0[n], out);
 		s += 3 * n_channels;
@@ -224,8 +232,8 @@ conv_s24_to_f32d_2s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 		s += 12 * n_channels;
 	}
 	for(; n < n_samples; n++) {
-		out[0] = _mm_cvtsi32_ss(out[0], read_s24(s));
-		out[1] = _mm_cvtsi32_ss(out[1], read_s24(s+3));
+		out[0] = _mm_cvtsi32_ss(factor, read_s24(s));
+		out[1] = _mm_cvtsi32_ss(factor, read_s24(s+3));
 		out[0] = _mm_mul_ss(out[0], factor);
 		out[1] = _mm_mul_ss(out[1], factor);
 		_mm_store_ss(&d0[n], out[0]);
@@ -305,10 +313,10 @@ conv_s24_to_f32d_4s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 		s += 12 * n_channels;
 	}
 	for(; n < n_samples; n++) {
-		out[0] = _mm_cvtsi32_ss(out[0], read_s24(s));
-		out[1] = _mm_cvtsi32_ss(out[1], read_s24(s+3));
-		out[2] = _mm_cvtsi32_ss(out[2], read_s24(s+6));
-		out[3] = _mm_cvtsi32_ss(out[3], read_s24(s+9));
+		out[0] = _mm_cvtsi32_ss(factor, read_s24(s));
+		out[1] = _mm_cvtsi32_ss(factor, read_s24(s+3));
+		out[2] = _mm_cvtsi32_ss(factor, read_s24(s+6));
+		out[3] = _mm_cvtsi32_ss(factor, read_s24(s+9));
 		out[0] = _mm_mul_ss(out[0], factor);
 		out[1] = _mm_mul_ss(out[1], factor);
 		out[2] = _mm_mul_ss(out[2], factor);
@@ -398,10 +406,10 @@ conv_s32_to_f32d_4s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 	}
 	for(; n < n_samples; n++) {
 		__m128 out[4], factor = _mm_set1_ps(1.0f / S24_SCALE);
-		out[0] = _mm_cvtsi32_ss(out[0], s[0]>>8);
-		out[1] = _mm_cvtsi32_ss(out[1], s[1]>>8);
-		out[2] = _mm_cvtsi32_ss(out[2], s[2]>>8);
-		out[3] = _mm_cvtsi32_ss(out[3], s[3]>>8);
+		out[0] = _mm_cvtsi32_ss(factor, s[0]>>8);
+		out[1] = _mm_cvtsi32_ss(factor, s[1]>>8);
+		out[2] = _mm_cvtsi32_ss(factor, s[2]>>8);
+		out[3] = _mm_cvtsi32_ss(factor, s[3]>>8);
 		out[0] = _mm_mul_ss(out[0], factor);
 		out[1] = _mm_mul_ss(out[1], factor);
 		out[2] = _mm_mul_ss(out[2], factor);
@@ -459,8 +467,8 @@ conv_s32_to_f32d_2s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 	}
 	for(; n < n_samples; n++) {
 		__m128 out[2], factor = _mm_set1_ps(1.0f / S24_SCALE);
-		out[0] = _mm_cvtsi32_ss(out[0], s[0]>>8);
-		out[1] = _mm_cvtsi32_ss(out[1], s[1]>>8);
+		out[0] = _mm_cvtsi32_ss(factor, s[0]>>8);
+		out[1] = _mm_cvtsi32_ss(factor, s[1]>>8);
 		out[0] = _mm_mul_ss(out[0], factor);
 		out[1] = _mm_mul_ss(out[1], factor);
 		_mm_store_ss(&d0[n], out[0]);
@@ -489,10 +497,10 @@ conv_s32_to_f32d_1s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 	for(n = 0; n < unrolled; n += 16) {
 		in[0] = _mm256_setr_m128i(
 				_mm256_i64gather_epi32(&s[ 0*n_channels], mask1, 4),
-				_mm256_i64gather_epi32(&s[ 0*n_channels], mask2, 4)),
+				_mm256_i64gather_epi32(&s[ 0*n_channels], mask2, 4));
 		in[1] = _mm256_setr_m128i(
 				_mm256_i64gather_epi32(&s[ 8*n_channels], mask1, 4),
-				_mm256_i64gather_epi32(&s[ 8*n_channels], mask2, 4)),
+				_mm256_i64gather_epi32(&s[ 8*n_channels], mask2, 4));
 
 		in[0] = _mm256_srai_epi32(in[0], 8);
 		in[1] = _mm256_srai_epi32(in[1], 8);
@@ -510,7 +518,7 @@ conv_s32_to_f32d_1s_avx2(void *data, void * SPA_RESTRICT dst[], const void * SPA
 	}
 	for(; n < n_samples; n++) {
 		__m128 out, factor = _mm_set1_ps(1.0f / S24_SCALE);
-		out = _mm_cvtsi32_ss(out, s[0]>>8);
+		out = _mm_cvtsi32_ss(factor, s[0]>>8);
 		out = _mm_mul_ss(out, factor);
 		_mm_store_ss(&d0[n], out);
 		s += n_channels;
@@ -542,7 +550,7 @@ conv_f32d_to_s32_1s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 	__m128 in[1];
 	__m128i out[4];
 	__m128 scale = _mm_set1_ps(S32_SCALE);
-	__m128 int_min = _mm_set1_ps(S32_MIN);
+	__m128 int_max = _mm_set1_ps(S32_MAX);
 
 	if (SPA_IS_ALIGNED(s0, 16))
 		unrolled = n_samples & ~3;
@@ -551,7 +559,7 @@ conv_f32d_to_s32_1s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 
 	for(n = 0; n < unrolled; n += 4) {
 		in[0] = _mm_mul_ps(_mm_load_ps(&s0[n]), scale);
-		in[0] = _mm_min_ps(in[0], int_min);
+		in[0] = _mm_min_ps(in[0], int_max);
 		out[0] = _mm_cvtps_epi32(in[0]);
 		out[1] = _mm_shuffle_epi32(out[0], _MM_SHUFFLE(0, 3, 2, 1));
 		out[2] = _mm_shuffle_epi32(out[0], _MM_SHUFFLE(1, 0, 3, 2));
@@ -566,7 +574,7 @@ conv_f32d_to_s32_1s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 	for(; n < n_samples; n++) {
 		in[0] = _mm_load_ss(&s0[n]);
 		in[0] = _mm_mul_ss(in[0], scale);
-		in[0] = _mm_min_ss(in[0], int_min);
+		in[0] = _mm_min_ss(in[0], int_max);
 		*d = _mm_cvtss_si32(in[0]);
 		d += n_channels;
 	}
@@ -582,7 +590,7 @@ conv_f32d_to_s32_2s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 	__m256 in[2];
 	__m256i out[2], t[2];
 	__m256 scale = _mm256_set1_ps(S32_SCALE);
-	__m256 int_min = _mm256_set1_ps(S32_MIN);
+	__m256 int_max = _mm256_set1_ps(S32_MAX);
 
 	if (SPA_IS_ALIGNED(s0, 32) &&
 	    SPA_IS_ALIGNED(s1, 32))
@@ -594,8 +602,8 @@ conv_f32d_to_s32_2s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		in[0] = _mm256_mul_ps(_mm256_load_ps(&s0[n]), scale);
 		in[1] = _mm256_mul_ps(_mm256_load_ps(&s1[n]), scale);
 
-		in[0] = _mm256_min_ps(in[0], int_min);
-		in[1] = _mm256_min_ps(in[1], int_min);
+		in[0] = _mm256_min_ps(in[0], int_max);
+		in[1] = _mm256_min_ps(in[1], int_max);
 
 		out[0] = _mm256_cvtps_epi32(in[0]);	/* a0 a1 a2 a3 a4 a5 a6 a7 */
 		out[1] = _mm256_cvtps_epi32(in[1]);	/* b0 b1 b2 b3 b4 b5 b6 b7 */
@@ -628,7 +636,7 @@ conv_f32d_to_s32_2s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		__m128 in[2];
 		__m128i out[2];
 		__m128 scale = _mm_set1_ps(S32_SCALE);
-		__m128 int_min = _mm_set1_ps(S32_MIN);
+		__m128 int_max = _mm_set1_ps(S32_MAX);
 
 		in[0] = _mm_load_ss(&s0[n]);
 		in[1] = _mm_load_ss(&s1[n]);
@@ -636,7 +644,7 @@ conv_f32d_to_s32_2s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		in[0] = _mm_unpacklo_ps(in[0], in[1]);
 
 		in[0] = _mm_mul_ps(in[0], scale);
-		in[0] = _mm_min_ps(in[0], int_min);
+		in[0] = _mm_min_ps(in[0], int_max);
 		out[0] = _mm_cvtps_epi32(in[0]);
 		_mm_storel_epi64((__m128i*)d, out[0]);
 		d += n_channels;
@@ -653,7 +661,7 @@ conv_f32d_to_s32_4s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 	__m256 in[4];
 	__m256i out[4], t[4];
 	__m256 scale = _mm256_set1_ps(S32_SCALE);
-	__m256 int_min = _mm256_set1_ps(S32_MIN);
+	__m256 int_max = _mm256_set1_ps(S32_MAX);
 
 	if (SPA_IS_ALIGNED(s0, 32) &&
 	    SPA_IS_ALIGNED(s1, 32) &&
@@ -669,10 +677,10 @@ conv_f32d_to_s32_4s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		in[2] = _mm256_mul_ps(_mm256_load_ps(&s2[n]), scale);
 		in[3] = _mm256_mul_ps(_mm256_load_ps(&s3[n]), scale);
 
-		in[0] = _mm256_min_ps(in[0], int_min);
-		in[1] = _mm256_min_ps(in[1], int_min);
-		in[2] = _mm256_min_ps(in[2], int_min);
-		in[3] = _mm256_min_ps(in[3], int_min);
+		in[0] = _mm256_min_ps(in[0], int_max);
+		in[1] = _mm256_min_ps(in[1], int_max);
+		in[2] = _mm256_min_ps(in[2], int_max);
+		in[3] = _mm256_min_ps(in[3], int_max);
 
 		out[0] = _mm256_cvtps_epi32(in[0]); /* a0 a1 a2 a3 a4 a5 a6 a7 */
 		out[1] = _mm256_cvtps_epi32(in[1]); /* b0 b1 b2 b3 b4 b5 b6 b7 */
@@ -703,7 +711,7 @@ conv_f32d_to_s32_4s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		__m128 in[4];
 		__m128i out[4];
 		__m128 scale = _mm_set1_ps(S32_SCALE);
-		__m128 int_min = _mm_set1_ps(S32_MIN);
+		__m128 int_max = _mm_set1_ps(S32_MAX);
 
 		in[0] = _mm_load_ss(&s0[n]);
 		in[1] = _mm_load_ss(&s1[n]);
@@ -715,7 +723,7 @@ conv_f32d_to_s32_4s_avx2(void *data, void * SPA_RESTRICT dst, const void * SPA_R
 		in[0] = _mm_unpacklo_ps(in[0], in[1]);
 
 		in[0] = _mm_mul_ps(in[0], scale);
-		in[0] = _mm_min_ps(in[0], int_min);
+		in[0] = _mm_min_ps(in[0], int_max);
 		out[0] = _mm_cvtps_epi32(in[0]);
 		_mm_storeu_si128((__m128i*)d, out[0]);
 		d += n_channels;
