@@ -38,13 +38,13 @@ static const struct quality blackman_qualities[] = {
 	{ 16, 0.70, },
 	{ 24, 0.76, },
 	{ 32, 0.8, },
-	{ 48, 0.865, },                  /* default */
+	{ 48, 0.85, },                  /* default */
 	{ 64, 0.90, },
 	{ 80, 0.92, },
 	{ 96, 0.933, },
 	{ 128, 0.950, },
 	{ 144, 0.955, },
-	{ 160, 0.960, },
+	{ 160, 0.958, },
 	{ 192, 0.965, },
 	{ 256, 0.975, },
 	{ 896, 0.997, },
@@ -58,12 +58,23 @@ static inline double sinc(double x)
 	return sin(x) / x;
 }
 
-static inline double blackman(double x, double n_taps)
+#if 0
+static inline double window_blackman(double x, double n_taps)
 {
-	double w = 2.0 * x * M_PI / n_taps + M_PI;
-	return 0.3635819 - 0.4891775 * cos(w) +
-		0.1365995 * cos(2 * w) - 0.0106411 * cos(3 * w);
+	double alpha = 0.232;
+	x =  2.0 * M_PI * x / n_taps;
+	return (1.0 - alpha) / 2.0 + (1.0 / 2.0) * cos(x) +
+		(alpha / 2.0) * cos(2 * x);
 }
+#else
+static inline double window_cosh(double x, double n_taps)
+{
+	double R = 95.0;
+	double A = -325.1E-6 * (R * R) + 0.1677 * R - 3.149;
+	x =  2.0 * M_PI * x / n_taps;
+	return cosh(A * sqrt(1 - pow(x / M_PI, 2))) / cosh(A);
+}
+#endif
 
 static int build_filter(float *taps, uint32_t stride, uint32_t n_taps, uint32_t n_phases, double cutoff)
 {
@@ -75,7 +86,7 @@ static int build_filter(float *taps, uint32_t stride, uint32_t n_taps, uint32_t
 			/* exploit symmetry in filter taps */
 			taps[(n_phases - i) * stride + n_taps12 + j] =
 				taps[i * stride + (n_taps12 - j - 1)] =
-					cutoff * sinc(t * cutoff) * blackman(t, n_taps);
+					cutoff * sinc(t * cutoff) * window_cosh(t, n_taps);
 		}
 	}
 	return 0;
@@ -85,10 +96,15 @@ static void inner_product_c(float *d, const float * SPA_RESTRICT s,
 		const float * SPA_RESTRICT taps, uint32_t n_taps)
 {
 	float sum = 0.0f;
+#if 1
+	uint32_t i, j, nt2 = n_taps/2;
+	for (i = 0, j = n_taps-1; i < nt2; i++, j--)
+		sum += s[i] * taps[i] + s[j] * taps[j];
+#else
 	uint32_t i;
-
 	for (i = 0; i < n_taps; i++)
 		sum += s[i] * taps[i];
+#endif
 	*d = sum;
 }
 
@@ -98,11 +114,18 @@ static void inner_product_ip_c(float *d, const float * SPA_RESTRICT s,
 {
 	float sum[2] = { 0.0f, 0.0f };
 	uint32_t i;
-
+#if 1
+	uint32_t j, nt2 = n_taps/2;
+	for (i = 0, j = n_taps-1; i < nt2; i++, j--) {
+		sum[0] += s[i] * t0[i] + s[j] * t0[j];
+		sum[1] += s[i] * t1[i] + s[j] * t1[j];
+	}
+#else
 	for (i = 0; i < n_taps; i++) {
 		sum[0] += s[i] * t0[i];
 		sum[1] += s[i] * t1[i];
 	}
+#endif
 	*d = (sum[1] - sum[0]) * x + sum[0];
 }
 
@@ -110,27 +133,27 @@ MAKE_RESAMPLER_COPY(c);
 MAKE_RESAMPLER_FULL(c);
 MAKE_RESAMPLER_INTER(c);
 
+#define MAKE(fmt,copy,full,inter,...) \
+	{ SPA_AUDIO_FORMAT_ ##fmt, do_resample_ ##copy, #copy, \
+		do_resample_ ##full, #full, do_resample_ ##inter, #inter, __VA_ARGS__ }
+
 static struct resample_info resample_table[] =
 {
 #if defined (HAVE_NEON)
-	{ SPA_AUDIO_FORMAT_F32, SPA_CPU_FLAG_NEON,
-		do_resample_copy_c, do_resample_full_neon, do_resample_inter_neon },
+	MAKE(F32, copy_c, full_neon, inter_neon, SPA_CPU_FLAG_NEON),
 #endif
 #if defined(HAVE_AVX) && defined(HAVE_FMA)
-	{ SPA_AUDIO_FORMAT_F32, SPA_CPU_FLAG_AVX | SPA_CPU_FLAG_FMA3,
-		do_resample_copy_c, do_resample_full_avx, do_resample_inter_avx },
+	MAKE(F32, copy_c, full_avx, inter_avx, SPA_CPU_FLAG_AVX | SPA_CPU_FLAG_FMA3),
 #endif
 #if defined (HAVE_SSSE3)
-	{ SPA_AUDIO_FORMAT_F32, SPA_CPU_FLAG_SSSE3 | SPA_CPU_FLAG_SLOW_UNALIGNED,
-		do_resample_copy_c, do_resample_full_ssse3, do_resample_inter_ssse3 },
+	MAKE(F32, copy_c, full_ssse3, inter_ssse3, SPA_CPU_FLAG_SSSE3 | SPA_CPU_FLAG_SLOW_UNALIGNED),
 #endif
 #if defined (HAVE_SSE)
-	{ SPA_AUDIO_FORMAT_F32, SPA_CPU_FLAG_SSE,
-		do_resample_copy_c, do_resample_full_sse, do_resample_inter_sse },
+	MAKE(F32, copy_c, full_sse, inter_sse, SPA_CPU_FLAG_SSE),
 #endif
-	{ SPA_AUDIO_FORMAT_F32, 0,
-		do_resample_copy_c, do_resample_full_c, do_resample_inter_c },
+	MAKE(F32, copy_c, full_c, inter_c),
 };
+#undef MAKE
 
 #define MATCH_CPU_FLAGS(a,b)	((a) == 0 || ((a) & (b)) == a)
 static const struct resample_info *find_resample_info(uint32_t format, uint32_t cpu_flags)
@@ -186,12 +209,18 @@ static void impl_native_update_rate(struct resample *r, double rate)
 	data->inc = data->in_rate / data->out_rate;
 	data->frac = data->in_rate % data->out_rate;
 
-	if (data->in_rate == data->out_rate)
+	if (data->in_rate == data->out_rate) {
 		data->func = data->info->process_copy;
-	else if (rate == 1.0)
+		r->func_name = data->info->copy_name;
+	}
+	else if (rate == 1.0) {
 		data->func = data->info->process_full;
-	else
+		r->func_name = data->info->full_name;
+	}
+	else {
 		data->func = data->info->process_inter;
+		r->func_name = data->info->inter_name;
+	}
 
 	spa_log_trace_fp(r->log, "native %p: rate:%f in:%d out:%d phase:%d inc:%d frac:%d", r,
 			rate, data->in_rate, data->out_rate, data->phase, data->inc, data->frac);
@@ -261,10 +290,10 @@ static void impl_native_process(struct resample *r,
 		in = *in_len;
 		data->func(r, src, skip, &in, dst, out, out_len);
 
-		spa_log_trace_fp(r->log, "native %p: in:%d/%d out %d/%d",
-				r, *in_len, in, *out_len, out);
+		spa_log_trace_fp(r->log, "native %p: in:%d/%d out %d/%d skip:%d",
+				r, *in_len, in, *out_len, out, skip);
 
-		remain = *in_len - skip - in;
+		remain = *in_len - in;
 		if (remain > 0 && remain <= n_taps) {
 			/* not enough input data remaining for more output,
 			 * copy to history */
@@ -382,10 +411,9 @@ int resample_native_init(struct resample *r)
 	build_filter(d->filter, d->filter_stride, n_taps, n_phases, scale);
 
 	d->info = find_resample_info(SPA_AUDIO_FORMAT_F32, r->cpu_flags);
-	if (SPA_UNLIKELY(!d->info))
-	{
+	if (SPA_UNLIKELY(d->info == NULL)) {
 	    spa_log_error(r->log, "failed to find suitable resample format!");
-	    return -1;
+	    return -ENOTSUP;
 	}
 
 	spa_log_debug(r->log, "native %p: q:%d in:%d out:%d n_taps:%d n_phases:%d features:%08x:%08x",
