@@ -6,7 +6,7 @@
 #include <sys/mman.h>
 #include <fcntl.h>
 #include <string.h>
-#ifndef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__MidnightBSD__)
 #include <alloca.h>
 #endif
 #include <errno.h>
