@@ -66,7 +66,6 @@
 
 static int handle_packet(struct client *client, struct message *msg)
 {
-	struct impl * const impl = client->impl;
 	uint32_t command, tag;
 	int res = 0;
 
@@ -110,7 +109,7 @@ static int handle_packet(struct client *client, struct message *msg)
 	res = cmd->run(client, command, tag, msg);
 
 finish:
-	message_free(impl, msg, false, false);
+	message_free(msg, false, false);
 	if (res < 0)
 		reply_error(client, command, tag, res);
 
@@ -119,7 +118,6 @@ finish:
 
 static int handle_memblock(struct client *client, struct message *msg)
 {
-	struct impl * const impl = client->impl;
 	struct stream *stream;
 	uint32_t channel, flags, index;
 	int64_t offset, diff;
@@ -190,7 +188,7 @@ static int handle_memblock(struct client *client, struct message *msg)
 	stream_send_request(stream);
 
 finish:
-	message_free(impl, msg, false, false);
+	message_free(msg, false, false);
 	return res;
 }
 
@@ -226,7 +224,8 @@ static int do_read(struct client *client)
 			if (errno == EINTR)
 				continue;
 			res = -errno;
-			if (res != -EAGAIN && res != -EWOULDBLOCK)
+			if (res != -EAGAIN && res != -EWOULDBLOCK &&
+			    res != -EPIPE && res != -ECONNRESET)
 				pw_log_warn("recv client:%p res %zd: %m", client, r);
 			goto exit;
 		}
@@ -263,7 +262,7 @@ static int do_read(struct client *client)
 		}
 
 		if (client->message)
-			message_free(impl, client->message, false, false);
+			message_free(client->message, false, false);
 
 		client->message = message_alloc(impl, channel, length);
 	} else if (client->message &&
@@ -327,6 +326,7 @@ done:
 error:
 	switch (res) {
 	case -EPIPE:
+	case -ECONNRESET:
 		pw_log_info("server %p: client %p [%s] disconnected",
 			    client->server, client, client->name);
 		SPA_FALLTHROUGH;
@@ -400,6 +400,7 @@ on_connect(void *data, int fd, uint32_t mask)
 
 	client->props = pw_properties_new(
 			PW_KEY_CLIENT_API, "pipewire-pulse",
+			"config.ext", pw_properties_get(impl->props, "config.ext"),
 			NULL);
 	if (client->props == NULL)
 		goto error;
@@ -457,7 +458,7 @@ static int parse_unix_address(const char *address, struct sockaddr_storage *addr
 	if (address[0] != '/') {
 		char runtime_dir[PATH_MAX];
 
-		if ((res = get_runtime_dir(runtime_dir, sizeof(runtime_dir), "pulse")) < 0)
+		if ((res = get_runtime_dir(runtime_dir, sizeof(runtime_dir))) < 0)
 			return res;
 
 		res = snprintf(addr.sun_path, sizeof(addr.sun_path),
