@@ -35,6 +35,7 @@
 #include <limits.h>
 #include <math.h>
 #include <arpa/inet.h>
+#include <netinet/in.h>
 
 #include <openssl/err.h>
 #include <openssl/rand.h>
@@ -57,10 +58,21 @@
 
 #include <pipewire/impl.h>
 #include <pipewire/i18n.h>
+#include <pipewire/private.h>
 
 #include "module-raop/rtsp-client.h"
 
 /** \page page_module_raop_sink PipeWire Module: AirPlay Sink
+ *
+ * Creates a new Sink to stream to an Airplay device.
+ *
+ * ## Module Options
+ *
+ * ## Example configuration
+ *
+ * ## See also
+ *
+ * \ref page_module_raop_discover
  */
 
 #define NAME "raop-sink"
@@ -89,15 +101,17 @@ PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
 
 #define DEFAULT_FORMAT "S16"
 #define DEFAULT_RATE 44100
-#define DEFAULT_CHANNELS "2"
+#define DEFAULT_CHANNELS 2
 #define DEFAULT_POSITION "[ FL FR ]"
 
+#define DEFAULT_LATENCY (DEFAULT_RATE*2)
+
 #define MODULE_USAGE	"[ node.latency=<latency as fraction> ] "				\
 			"[ node.name=<name of the nodes> ] "					\
 			"[ node.description=<description of the nodes> ] "			\
 			"[ audio.format=<format, default:"DEFAULT_FORMAT"> ] "			\
-			"[ audio.rate=<sample rate, default: 48000> ] "				\
-			"[ audio.channels=<number of channels, default:"DEFAULT_CHANNELS"> ] "	\
+			"[ audio.rate=<sample rate, default: "SPA_STRINGIFY(DEFAULT_RATE)"> ] "			\
+			"[ audio.channels=<number of channels, default:"SPA_STRINGIFY(DEFAULT_CHANNELS)"> ] "	\
 			"[ audio.position=<channel map, default:"DEFAULT_POSITION"> ] "		\
 			"[ stream.props=<properties> ] "
 
@@ -131,7 +145,6 @@ struct impl {
 
 	struct pw_impl_module *module;
 	struct pw_loop *loop;
-	struct pw_work_queue *work;
 
 	struct spa_hook module_listener;
 
@@ -157,7 +170,6 @@ struct impl {
 	char *password;
 
 	unsigned int do_disconnect:1;
-	unsigned int unloading:1;
 
 	uint8_t key[AES_CHUNK_SIZE]; /* Key for aes-cbc */
 	uint8_t iv[AES_CHUNK_SIZE];  /* Initialization vector for cbc */
@@ -193,20 +205,6 @@ struct impl {
 	uint32_t filled;
 };
 
-static void do_unload_module(void *obj, void *data, int res, uint32_t id)
-{
-	struct impl *impl = data;
-	pw_impl_module_destroy(impl->module);
-}
-
-static void unload_module(struct impl *impl)
-{
-	if (!impl->unloading) {
-		impl->unloading = true;
-		pw_work_queue_add(impl->work, impl, 0, do_unload_module, impl);
-	}
-}
-
 static void stream_destroy(void *d)
 {
 	struct impl *impl = d;
@@ -432,7 +430,7 @@ static void playback_stream_process(void *d)
 	struct pw_buffer *buf;
 	struct spa_data *bd;
 	uint8_t *data;
-	uint32_t size;
+	uint32_t offs, size;
 
 	if ((buf = pw_stream_dequeue_buffer(impl->stream)) == NULL) {
 		pw_log_debug("out of buffers: %m");
@@ -440,8 +438,10 @@ static void playback_stream_process(void *d)
 	}
 
 	bd = &buf->buffer->datas[0];
-	data = SPA_PTROFF(bd->data, bd->chunk->offset, uint8_t);
-	size = bd->chunk->size;
+
+	offs = SPA_MIN(bd->chunk->offset, bd->maxsize);
+	size = SPA_MIN(bd->chunk->size, bd->maxsize - offs);
+	data = SPA_PTROFF(bd->data, offs, uint8_t);
 
 	while (size > 0 && impl->block_size > 0) {
 		uint32_t avail, to_fill;
@@ -596,6 +596,10 @@ on_timing_source_io(void *data, int fd, uint32_t mask)
 
 		received = ntp_now(CLOCK_MONOTONIC);
 		bytes = read(impl->timing_fd, packet, sizeof(packet));
+		if (bytes < 0) {
+			pw_log_debug("error reading timing packet: %m");
+			return;
+		}
 		if (bytes != sizeof(packet)) {
 			pw_log_warn("discarding short (%zd < %zd) timing packet",
 					bytes, sizeof(bytes));
@@ -621,7 +625,11 @@ on_control_source_io(void *data, int fd, uint32_t mask)
 		uint32_t hdr;
 		uint16_t seq, num;
 
-		bytes = read(impl->timing_fd, packet, sizeof(packet));
+		bytes = read(impl->control_fd, packet, sizeof(packet));
+		if (bytes < 0) {
+			pw_log_debug("error reading control packet: %m");
+			return;
+		}
 		if (bytes != sizeof(packet)) {
 			pw_log_warn("discarding short (%zd < %zd) control packet",
 					bytes, sizeof(bytes));
@@ -686,7 +694,9 @@ static void rtsp_record_reply(void *data, int status, const struct spa_dict *hea
 
 	if ((str = spa_dict_lookup(headers, "Audio-Latency")) != NULL) {
 		if (!spa_atou32(str, &impl->latency, 0))
-			impl->latency = 0;
+			impl->latency = DEFAULT_LATENCY;
+	} else {
+		impl->latency = DEFAULT_LATENCY;
 	}
 
 	spa_zero(latency);
@@ -1133,7 +1143,7 @@ static int rtsp_do_auth(struct impl *impl, const struct spa_dict *headers)
 				DEFAULT_USER_NAME, realm, nonce, resp);
 	}
 	else
-		return -EINVAL;
+		goto error;
 
 	pw_properties_setf(impl->headers, "Authorization", "%s %s",
 			tokens[0], auth);
@@ -1255,7 +1265,7 @@ static void stream_state_changed(void *d, enum pw_stream_state old,
 	switch (state) {
 	case PW_STREAM_STATE_ERROR:
 	case PW_STREAM_STATE_UNCONNECTED:
-		unload_module(impl);
+		pw_impl_module_schedule_destroy(impl->module);
 		break;
 	case PW_STREAM_STATE_PAUSED:
 		rtsp_do_flush(impl);
@@ -1389,7 +1399,7 @@ static void core_error(void *data, uint32_t id, int seq, int res, const char *me
 			id, seq, res, spa_strerror(res), message);
 
 	if (id == PW_ID_CORE && res == -EPIPE)
-		unload_module(impl);
+		pw_impl_module_schedule_destroy(impl->module);
 }
 
 static const struct pw_core_events core_events = {
@@ -1402,7 +1412,7 @@ static void core_destroy(void *d)
 	struct impl *impl = d;
 	spa_hook_remove(&impl->core_listener);
 	impl->core = NULL;
-	unload_module(impl);
+	pw_impl_module_schedule_destroy(impl->module);
 }
 
 static const struct pw_proxy_events core_proxy_events = {
@@ -1423,15 +1433,12 @@ static void impl_destroy(struct impl *impl)
 	pw_properties_free(impl->stream_props);
 	pw_properties_free(impl->props);
 	free(impl->password);
-	if (impl->work)
-		pw_work_queue_cancel(impl->work, impl, SPA_ID_INVALID);
 	free(impl);
 }
 
 static void module_destroy(void *data)
 {
 	struct impl *impl = data;
-	impl->unloading = true;
 	spa_hook_remove(&impl->module_listener);
 	impl_destroy(impl);
 }
@@ -1477,58 +1484,59 @@ static void parse_position(struct spa_audio_info_raw *info, const char *val, siz
 	}
 }
 
-static int parse_audio_info(struct impl *impl)
+static void parse_audio_info(const struct pw_properties *props, struct spa_audio_info_raw *info)
 {
-	struct pw_properties *props = impl->stream_props;
-	struct spa_audio_info_raw *info = &impl->info;
 	const char *str;
 
 	spa_zero(*info);
-
 	if ((str = pw_properties_get(props, PW_KEY_AUDIO_FORMAT)) == NULL)
 		str = DEFAULT_FORMAT;
 	info->format = format_from_name(str, strlen(str));
-	switch (info->format) {
-	case SPA_AUDIO_FORMAT_S8:
-	case SPA_AUDIO_FORMAT_U8:
-		impl->frame_size = 1;
-		break;
-	case SPA_AUDIO_FORMAT_S16:
-		impl->frame_size = 2;
-		break;
-	case SPA_AUDIO_FORMAT_S24:
-		impl->frame_size = 3;
-		break;
-	case SPA_AUDIO_FORMAT_S24_32:
-	case SPA_AUDIO_FORMAT_S32:
-	case SPA_AUDIO_FORMAT_F32:
-		impl->frame_size = 4;
-		break;
-	case SPA_AUDIO_FORMAT_F64:
-		impl->frame_size = 8;
-		break;
-	default:
-		pw_log_error("unsupported format '%s'", str);
-		return -EINVAL;
-	}
-	info->rate = pw_properties_get_uint32(props, PW_KEY_AUDIO_RATE, DEFAULT_RATE);
-	if (info->rate == 0) {
-		pw_log_error("invalid rate '%s'", str);
-		return -EINVAL;
-	}
-	if ((str = pw_properties_get(props, PW_KEY_AUDIO_CHANNELS)) == NULL)
-		str = DEFAULT_CHANNELS;
-	info->channels = atoi(str);
-	if ((str = pw_properties_get(props, SPA_KEY_AUDIO_POSITION)) == NULL)
-		str = DEFAULT_POSITION;
-	parse_position(info, str, strlen(str));
-	if (info->channels == 0) {
-		pw_log_error("invalid channels '%s'", str);
-		return -EINVAL;
-	}
-	impl->frame_size *= info->channels;
 
-	return 0;
+	info->rate = pw_properties_get_uint32(props, PW_KEY_AUDIO_RATE, info->rate);
+	if (info->rate == 0)
+		info->rate = DEFAULT_RATE;
+
+	info->channels = pw_properties_get_uint32(props, PW_KEY_AUDIO_CHANNELS, info->channels);
+	info->channels = SPA_MIN(info->channels, SPA_AUDIO_MAX_CHANNELS);
+	if ((str = pw_properties_get(props, SPA_KEY_AUDIO_POSITION)) != NULL)
+		parse_position(info, str, strlen(str));
+	if (info->channels == 0)
+		parse_position(info, DEFAULT_POSITION, strlen(DEFAULT_POSITION));
+}
+
+static int calc_frame_size(struct spa_audio_info_raw *info)
+{
+	int res = info->channels;
+	switch (info->format) {
+	case SPA_AUDIO_FORMAT_U8:
+	case SPA_AUDIO_FORMAT_S8:
+	case SPA_AUDIO_FORMAT_ALAW:
+	case SPA_AUDIO_FORMAT_ULAW:
+		return res;
+	case SPA_AUDIO_FORMAT_S16:
+	case SPA_AUDIO_FORMAT_S16_OE:
+	case SPA_AUDIO_FORMAT_U16:
+		return res * 2;
+	case SPA_AUDIO_FORMAT_S24:
+	case SPA_AUDIO_FORMAT_S24_OE:
+	case SPA_AUDIO_FORMAT_U24:
+		return res * 3;
+	case SPA_AUDIO_FORMAT_S24_32:
+	case SPA_AUDIO_FORMAT_S24_32_OE:
+	case SPA_AUDIO_FORMAT_S32:
+	case SPA_AUDIO_FORMAT_S32_OE:
+	case SPA_AUDIO_FORMAT_U32:
+	case SPA_AUDIO_FORMAT_U32_OE:
+	case SPA_AUDIO_FORMAT_F32:
+	case SPA_AUDIO_FORMAT_F32_OE:
+		return res * 4;
+	case SPA_AUDIO_FORMAT_F64:
+	case SPA_AUDIO_FORMAT_F64_OE:
+		return res * 8;
+	default:
+		return 0;
+	}
 }
 
 static void copy_props(struct impl *impl, struct pw_properties *props, const char *key)
@@ -1546,6 +1554,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	struct pw_context *context = pw_impl_module_get_context(module);
 	struct pw_properties *props = NULL;
 	uint32_t id = pw_global_get_id(pw_impl_module_get_global(module));
+	uint32_t pid = getpid();
 	struct impl *impl;
 	const char *str;
 	int res;
@@ -1582,15 +1591,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	impl->module = module;
 	impl->context = context;
 	impl->loop = pw_context_get_main_loop(context);
-	impl->work = pw_context_get_work_queue(context);
-	if (impl->work == NULL) {
-		res = -errno;
-		pw_log_error( "can't get work queue: %m");
-		goto error;
-	}
 
-	if (pw_properties_get(props, PW_KEY_NODE_GROUP) == NULL)
-		pw_properties_set(props, PW_KEY_NODE_GROUP, "pipewire.dummy");
 	if (pw_properties_get(props, PW_KEY_NODE_VIRTUAL) == NULL)
 		pw_properties_set(props, PW_KEY_NODE_VIRTUAL, "true");
 
@@ -1598,7 +1599,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		pw_properties_set(props, PW_KEY_MEDIA_CLASS, "Audio/Sink");
 
 	if (pw_properties_get(props, PW_KEY_NODE_NAME) == NULL)
-		pw_properties_setf(props, PW_KEY_NODE_NAME, "raop-sink-%u", id);
+		pw_properties_setf(props, PW_KEY_NODE_NAME, "raop-sink-%u-%u", pid, id);
 	if (pw_properties_get(props, PW_KEY_NODE_DESCRIPTION) == NULL)
 		pw_properties_set(props, PW_KEY_NODE_DESCRIPTION,
 				pw_properties_get(props, PW_KEY_NODE_NAME));
@@ -1619,8 +1620,13 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	copy_props(impl, props, PW_KEY_NODE_VIRTUAL);
 	copy_props(impl, props, PW_KEY_MEDIA_CLASS);
 
-	if ((res = parse_audio_info(impl)) < 0) {
-		pw_log_error( "can't parse audio format");
+	parse_audio_info(impl->stream_props, &impl->info);
+
+	impl->frame_size = calc_frame_size(&impl->info);
+	if (impl->frame_size == 0) {
+		pw_log_error("unsupported audio format:%d channels:%d",
+				impl->info.format, impl->info.channels);
+		res = -EINVAL;
 		goto error;
 	}
 
@@ -1632,6 +1638,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		impl->protocol = PROTO_TCP;
 	else {
 		pw_log_error( "can't handle transport %s", str);
+		res = -EINVAL;
 		goto error;
 	}
 
@@ -1643,6 +1650,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		impl->encryption = CRYPTO_RSA;
 	else {
 		pw_log_error( "can't handle encryption type %s", str);
+		res = -EINVAL;
 		goto error;
 	}
 
@@ -1652,6 +1660,7 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		impl->codec = CODEC_PCM;
 	else {
 		pw_log_error( "can't handle codec type %s", str);
+		res = -EINVAL;
 		goto error;
 	}
 	str = pw_properties_get(props, "raop.password");
