@@ -172,6 +172,7 @@ struct stream {
 	unsigned int driving:1;
 	unsigned int using_trigger:1;
 	unsigned int trigger:1;
+	int in_set_control;
 };
 
 static int get_param_index(uint32_t id)
@@ -543,10 +544,34 @@ static int impl_set_param(void *object, uint32_t id, uint32_t flags, const struc
 	if (id != SPA_PARAM_Props)
 		return -ENOTSUP;
 
-	pw_stream_emit_param_changed(stream, id, param);
+	if (impl->in_set_control == 0)
+		pw_stream_emit_param_changed(stream, id, param);
+
 	return 0;
 }
 
+static inline uint32_t update_requested(struct stream *impl)
+{
+	uint32_t index, id, res = 0;
+	struct buffer *buffer;
+	struct spa_io_rate_match *r = impl->rate_match;
+
+	if (spa_ringbuffer_get_read_index(&impl->dequeued.ring, &index) < 1)
+		return 0;
+
+	id = impl->dequeued.ids[index & MASK_BUFFERS];
+	buffer = &impl->buffers[id];
+	if (r) {
+		buffer->this.requested = r->size;
+		res = r->size > 0 ? 1 : 0;
+	} else {
+		buffer->this.requested = impl->quantum;
+		res = 1;
+	}
+	pw_log_trace_fp("%p: update buffer:%u size:%u", impl, id, r->size);
+	return res;
+}
+
 static int impl_send_command(void *object, const struct spa_command *command)
 {
 	struct stream *impl = object;
@@ -574,8 +599,10 @@ static int impl_send_command(void *object, const struct spa_command *command)
 
 			if (impl->direction == SPA_DIRECTION_INPUT)
 				impl->io->status = SPA_STATUS_NEED_DATA;
-			else if (!impl->process_rt && !impl->driving)
-				call_process(impl);
+			else if (!impl->process_rt && !impl->driving) {
+				if (update_requested(impl) > 0)
+					call_process(impl);
+			}
 
 			stream_set_state(stream, PW_STREAM_STATE_STREAMING, NULL);
 		}
@@ -1029,12 +1056,13 @@ again:
 		if (!impl->process_rt && (recycled || res == SPA_STATUS_NEED_DATA)) {
 			/* not realtime and we have a free buffer, trigger process so that we have
 			 * data in the next round. */
-			if (spa_ringbuffer_get_read_index(&impl->dequeued.ring, &index) > 0)
+			if (update_requested(impl) > 0)
 				call_process(impl);
 		} else if (res == SPA_STATUS_NEED_DATA) {
 			/* realtime and we don't have a buffer, trigger process and try
 			 * again when there is something in the queue now */
-			call_process(impl);
+			if (update_requested(impl) > 0)
+				call_process(impl);
 			if (impl->draining ||
 			    spa_ringbuffer_get_read_index(&impl->queued.ring, &index) > 0)
 				goto again;
@@ -1127,7 +1155,7 @@ static int node_event_param(void *object, int seq,
 		struct control *c;
 		const struct spa_pod *type, *pod;
 		uint32_t iid, choice, n_vals, container = SPA_ID_INVALID;
-		float *vals, bool_range[3] = { 1.0, 0.0, 1.0 };
+		float *vals, bool_range[3] = { 1.0, 0.0, 1.0 }, dbl[3];
 
 		if (spa_pod_parse_object(param,
 					SPA_TYPE_OBJECT_PropInfo, NULL,
@@ -1147,42 +1175,55 @@ static int node_event_param(void *object, int seq,
 
 		if (spa_pod_parse_object(c->info,
 					SPA_TYPE_OBJECT_PropInfo, NULL,
-					SPA_PROP_INFO_name, SPA_POD_String(&c->control.name),
+					SPA_PROP_INFO_description, SPA_POD_OPT_String(&c->control.name),
 					SPA_PROP_INFO_type, SPA_POD_PodChoice(&type),
 					SPA_PROP_INFO_container, SPA_POD_OPT_Id(&container)) < 0) {
 			free(c);
 			return -EINVAL;
 		}
 
-		spa_list_append(&stream->controls, &c->link);
-
 		pod = spa_pod_get_values(type, &n_vals, &choice);
 
 		c->type = SPA_POD_TYPE(pod);
 		if (spa_pod_is_float(pod))
 			vals = SPA_POD_BODY(pod);
+		else if (spa_pod_is_double(pod)) {
+			double *v = SPA_POD_BODY(pod);
+			dbl[0] = v[0];
+			if (n_vals > 1)
+				dbl[1] = v[1];
+			if (n_vals > 2)
+				dbl[2] = v[2];
+			vals = dbl;
+		}
 		else if (spa_pod_is_bool(pod) && n_vals > 0) {
 			choice = SPA_CHOICE_Range;
 			vals = bool_range;
 			vals[0] = SPA_POD_VALUE(struct spa_pod_bool, pod);
 			n_vals = 3;
 		}
-		else
+		else {
+			free(c);
 			return -ENOTSUP;
+		}
 
 		c->container = container != SPA_ID_INVALID ? container : c->type;
 
 		switch (choice) {
 		case SPA_CHOICE_None:
-			if (n_vals < 1)
+			if (n_vals < 1) {
+				free(c);
 				return -EINVAL;
+			}
 			c->control.n_values = 1;
 			c->control.max_values = 1;
 			c->control.values[0] = c->control.def = c->control.min = c->control.max = vals[0];
 			break;
 		case SPA_CHOICE_Range:
-			if (n_vals < 3)
+			if (n_vals < 3) {
+				free(c);
 				return -EINVAL;
+			}
 			c->control.n_values = 1;
 			c->control.max_values = 1;
 			c->control.values[0] = vals[0];
@@ -1191,10 +1232,12 @@ static int node_event_param(void *object, int seq,
 			c->control.max = vals[2];
 			break;
 		default:
+			free(c);
 			return -ENOTSUP;
 		}
 
 		c->id = iid;
+		spa_list_append(&stream->controls, &c->link);
 		pw_log_debug("%p: add control %d (%s) container:%d (def:%f min:%f max:%f)",
 				stream, c->id, c->control.name, c->container,
 				c->control.def, c->control.min, c->control.max);
@@ -1206,6 +1249,7 @@ static int node_event_param(void *object, int seq,
 		struct spa_pod_object *obj = (struct spa_pod_object *) param;
 		union {
 			float f;
+			double d;
 			bool b;
 		} value;
 		float *values;
@@ -1225,6 +1269,13 @@ static int node_event_param(void *object, int seq,
 				n_values = 1;
 				values = &value.f;
 				break;
+			case SPA_TYPE_Double:
+				if (spa_pod_get_double(&prop->value, &value.d) < 0)
+					continue;
+				n_values = 1;
+				value.f = value.d;
+				values = &value.f;
+				break;
 			case SPA_TYPE_Bool:
 				if (spa_pod_get_bool(&prop->value, &value.b) < 0)
 					continue;
@@ -1264,9 +1315,9 @@ static int node_event_param(void *object, int seq,
 	return 0;
 }
 
-static void node_event_info(void *object, const struct pw_node_info *info)
+static void node_event_info(void *data, const struct pw_node_info *info)
 {
-	struct pw_stream *stream = object;
+	struct pw_stream *stream = data;
 	struct stream *impl = SPA_CONTAINER_OF(stream, struct stream, this);
 	uint32_t i;
 
@@ -1294,9 +1345,9 @@ static const struct pw_impl_node_events node_events = {
 	.info_changed = node_event_info,
 };
 
-static void on_core_error(void *object, uint32_t id, int seq, int res, const char *message)
+static void on_core_error(void *data, uint32_t id, int seq, int res, const char *message)
 {
-	struct pw_stream *stream = object;
+	struct pw_stream *stream = data;
 
 	pw_log_debug("%p: error id:%u seq:%d res:%d (%s): %s", stream,
 			id, seq, res, spa_strerror(res), message);
@@ -1328,6 +1379,22 @@ static const struct pw_context_driver_events context_events = {
 	.drained = context_drained,
 };
 
+struct match {
+	struct pw_stream *stream;
+	int count;
+};
+#define MATCH_INIT(s) (struct match){ .stream = s }
+
+static int execute_match(void *data, const char *location, const char *action,
+		const char *val, size_t len)
+{
+	struct match *match = data;
+	struct pw_stream *this = match->stream;
+	if (spa_streq(action, "update-props"))
+		match->count += pw_properties_update_string(this->properties, val, len);
+	return 1;
+}
+
 static struct stream *
 stream_new(struct pw_context *context, const char *name,
 		struct pw_properties *props, const struct pw_properties *extra)
@@ -1335,6 +1402,7 @@ stream_new(struct pw_context *context, const char *name,
 	struct stream *impl;
 	struct pw_stream *this;
 	const char *str;
+	struct match match;
 	int res;
 
 	impl = calloc(1, sizeof(struct stream));
@@ -1360,23 +1428,17 @@ stream_new(struct pw_context *context, const char *name,
 		res = -errno;
 		goto error_properties;
 	}
+	spa_hook_list_init(&impl->hooks);
+	this->properties = props;
+
 	pw_context_conf_update_props(context, "stream.properties", props);
 
-	if (pw_properties_get(props, PW_KEY_STREAM_IS_LIVE) == NULL)
-		pw_properties_set(props, PW_KEY_STREAM_IS_LIVE, "true");
+	match = MATCH_INIT(this);
+	pw_context_conf_section_match_rules(context, "stream.rules",
+			&this->properties->dict, execute_match, &match);
 
-	if (pw_properties_get(props, PW_KEY_NODE_NAME) == NULL && extra) {
-		str = pw_properties_get(extra, PW_KEY_APP_NAME);
-		if (str == NULL)
-			str = pw_properties_get(extra, PW_KEY_APP_PROCESS_BINARY);
-		if (str == NULL)
-			str = name;
-		pw_properties_set(props, PW_KEY_NODE_NAME, str);
-	}
-	if ((str = getenv("PIPEWIRE_LATENCY")) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
-	if ((str = getenv("PIPEWIRE_RATE")) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_RATE, str);
+	if ((str = getenv("PIPEWIRE_PROPS")) != NULL)
+		pw_properties_update_string(props, str, strlen(str));
 	if ((str = getenv("PIPEWIRE_QUANTUM")) != NULL) {
 		struct spa_fraction q;
 		if (sscanf(str, "%u/%u", &q.num, &q.denom) == 2 && q.denom != 0) {
@@ -1386,9 +1448,21 @@ stream_new(struct pw_context *context, const char *name,
 					"%u/%u", q.num, q.denom);
 		}
 	}
+	if ((str = getenv("PIPEWIRE_LATENCY")) != NULL)
+		pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
+	if ((str = getenv("PIPEWIRE_RATE")) != NULL)
+		pw_properties_set(props, PW_KEY_NODE_RATE, str);
 
-	spa_hook_list_init(&impl->hooks);
-	this->properties = props;
+	if (pw_properties_get(props, PW_KEY_STREAM_IS_LIVE) == NULL)
+		pw_properties_set(props, PW_KEY_STREAM_IS_LIVE, "true");
+	if (pw_properties_get(props, PW_KEY_NODE_NAME) == NULL && extra) {
+		str = pw_properties_get(extra, PW_KEY_APP_NAME);
+		if (str == NULL)
+			str = pw_properties_get(extra, PW_KEY_APP_PROCESS_BINARY);
+		if (str == NULL)
+			str = name;
+		pw_properties_set(props, PW_KEY_NODE_NAME, str);
+	}
 
 	this->name = name ? strdup(name) : NULL;
 	this->node_id = SPA_ID_INVALID;
@@ -1598,14 +1672,21 @@ int pw_stream_update_properties(struct pw_stream *stream, const struct spa_dict
 {
 	struct stream *impl = SPA_CONTAINER_OF(stream, struct stream, this);
 	int changed, res = 0;
+	struct match match;
 
 	changed = pw_properties_update(stream->properties, dict);
-
 	if (!changed)
 		return 0;
 
+	match = MATCH_INIT(stream);
+	pw_context_conf_section_match_rules(impl->context, "stream.rules",
+			&stream->properties->dict, execute_match, &match);
+
 	if (impl->node)
-		res = pw_impl_node_update_properties(impl->node, dict);
+		res = pw_impl_node_update_properties(impl->node,
+				match.count == 0 ?
+					dict :
+					&stream->properties->dict);
 
 	return res;
 }
@@ -1699,7 +1780,6 @@ pw_stream_connect(struct pw_stream *stream,
 	struct stream *impl = SPA_CONTAINER_OF(stream, struct stream, this);
 	struct pw_impl_factory *factory;
 	struct pw_properties *props = NULL;
-	struct pw_impl_node *follower;
 	const char *str;
 	uint32_t i;
 	int res;
@@ -1792,6 +1872,9 @@ pw_stream_connect(struct pw_stream *stream,
 	}
 	if (flags & PW_STREAM_FLAG_DRIVER)
 		pw_properties_set(stream->properties, PW_KEY_NODE_DRIVER, "true");
+	if ((pw_properties_get(stream->properties, PW_KEY_NODE_WANT_DRIVER) == NULL))
+		pw_properties_set(stream->properties, PW_KEY_NODE_WANT_DRIVER, "true");
+
 	if (flags & PW_STREAM_FLAG_EXCLUSIVE)
 		pw_properties_set(stream->properties, PW_KEY_NODE_EXCLUSIVE, "true");
 	if (flags & PW_STREAM_FLAG_DONT_RECONNECT)
@@ -1810,6 +1893,7 @@ pw_stream_connect(struct pw_stream *stream,
 				direction == PW_DIRECTION_INPUT ? "Input" : "Output",
 				media_type ? media_type : get_media_class(impl));
 	}
+
 	if ((str = pw_properties_get(stream->properties, PW_KEY_FORMAT_DSP)) != NULL)
 		pw_properties_set(impl->port_props, PW_KEY_FORMAT_DSP, str);
 	else if (impl->media_type == SPA_MEDIA_TYPE_application &&
@@ -1844,14 +1928,6 @@ pw_stream_connect(struct pw_stream *stream,
 		pw_properties_set(props, "channelmix.normalize", "true");
 	}
 
-	follower = pw_context_create_node(impl->context, pw_properties_copy(props), 0);
-	if (follower == NULL) {
-		res = -errno;
-		goto error_node;
-	}
-
-	pw_impl_node_set_implementation(follower, &impl->impl_node);
-
 	if (impl->media_type == SPA_MEDIA_TYPE_audio) {
 		factory = pw_context_find_factory(impl->context, "adapter");
 		if (factory == NULL) {
@@ -1859,7 +1935,8 @@ pw_stream_connect(struct pw_stream *stream,
 			res = -ENOENT;
 			goto error_node;
 		}
-		pw_properties_setf(props, "adapt.follower.node", "pointer:%p", follower);
+		pw_properties_setf(props, "adapt.follower.spa-node", "pointer:%p",
+				&impl->impl_node);
 		pw_properties_set(props, "object.register", "false");
 		impl->node = pw_impl_factory_create_object(factory,
 				NULL,
@@ -1873,9 +1950,13 @@ pw_stream_connect(struct pw_stream *stream,
 			goto error_node;
 		}
 	} else {
-		impl->node = follower;
-		pw_properties_free(props);
+		impl->node = pw_context_create_node(impl->context, props, 0);
 		props = NULL;
+		if (impl->node == NULL) {
+			res = -errno;
+			goto error_node;
+		}
+		pw_impl_node_set_implementation(impl->node, &impl->impl_node);
 	}
 	pw_impl_node_set_active(impl->node,
 			!SPA_FLAG_IS_SET(impl->flags, PW_STREAM_FLAG_INACTIVE));
@@ -2002,6 +2083,9 @@ int pw_stream_set_control(struct pw_stream *stream, uint32_t id, uint32_t n_valu
 	struct spa_pod *pod;
 	struct control *c;
 
+	if (impl->node == NULL)
+		return -EIO;
+
 	va_start(varargs, values);
 
 	spa_pod_builder_push_object(&b, &f[0], SPA_TYPE_OBJECT_Props, SPA_PARAM_Props);
@@ -2014,6 +2098,9 @@ int pw_stream_set_control(struct pw_stream *stream, uint32_t id, uint32_t n_valu
 			case SPA_TYPE_Float:
 				spa_pod_builder_float(&b, values[0]);
 				break;
+			case SPA_TYPE_Double:
+				spa_pod_builder_double(&b, values[0]);
+				break;
 			case SPA_TYPE_Bool:
 				spa_pod_builder_bool(&b, values[0] < 0.5 ? false : true);
 				break;
@@ -2038,7 +2125,9 @@ int pw_stream_set_control(struct pw_stream *stream, uint32_t id, uint32_t n_valu
 
 	va_end(varargs);
 
+	impl->in_set_control++;
 	pw_impl_node_set_param(impl->node, SPA_PARAM_Props, 0, pod);
+	impl->in_set_control--;
 
 	return 0;
 }
@@ -2070,17 +2159,32 @@ int pw_stream_set_active(struct pw_stream *stream, bool active)
 	return 0;
 }
 
+struct old_time {
+	int64_t now;
+	struct spa_fraction rate;
+	uint64_t ticks;
+	int64_t delay;
+	uint64_t queued;
+};
+
 SPA_EXPORT
 int pw_stream_get_time(struct pw_stream *stream, struct pw_time *time)
+{
+	return pw_stream_get_time_n(stream, time, sizeof(struct old_time));
+}
+
+SPA_EXPORT
+int pw_stream_get_time_n(struct pw_stream *stream, struct pw_time *time, size_t size)
 {
 	struct stream *impl = SPA_CONTAINER_OF(stream, struct stream, this);
 	uintptr_t seq1, seq2;
-	uint32_t rate_queued;
+	uint32_t buffered, quantum, index;
 
 	do {
 		seq1 = SEQ_READ(impl->seq);
-		*time = impl->time;
-		rate_queued = impl->rate_queued;
+		memcpy(time, &impl->time, SPA_MIN(size, sizeof(struct pw_time)));
+		buffered = impl->rate_queued;
+		quantum = impl->quantum;
 		seq2 = SEQ_READ(impl->seq);
 	} while (!SEQ_READ_SUCCESS(seq1, seq2));
 
@@ -2089,19 +2193,23 @@ int pw_stream_get_time(struct pw_stream *stream, struct pw_time *time)
 	else
 		time->queued = (int64_t)(impl->queued.incount - time->queued);
 
-	time->queued += rate_queued;
-
-	time->delay += ((impl->latency.min_quantum + impl->latency.max_quantum) / 2) * impl->quantum;
+	time->delay += ((impl->latency.min_quantum + impl->latency.max_quantum) / 2) * quantum;
 	time->delay += (impl->latency.min_rate + impl->latency.max_rate) / 2;
 	time->delay += ((impl->latency.min_ns + impl->latency.max_ns) / 2) * time->rate.denom / SPA_NSEC_PER_SEC;
 
+	if (size >= offsetof(struct pw_time, queued_buffers))
+		time->buffered = buffered;
+	if (size >= offsetof(struct pw_time, avail_buffers))
+		time->queued_buffers = spa_ringbuffer_get_read_index(&impl->queued.ring, &index);
+	if (size >= sizeof(struct pw_time))
+		time->avail_buffers = spa_ringbuffer_get_read_index(&impl->dequeued.ring, &index);
+
 	pw_log_trace_fp("%p: %"PRIi64" %"PRIi64" %"PRIu64" %d/%d %"PRIu64" %"
 			PRIu64" %"PRIu64" %"PRIu64" %"PRIu64, stream,
 			time->now, time->delay, time->ticks,
 			time->rate.num, time->rate.denom, time->queued,
 			impl->dequeued.outcount, impl->dequeued.incount,
 			impl->queued.outcount, impl->queued.incount);
-
 	return 0;
 }
 
